/*
Eva library C++ (www.elxala.de)
Copyright (C) 2005  Alejandro Xalabarder Aulet

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
   Cadena.cpp
   Alejandro Xalabarder
   03.07.2003

   10.11.2007 fix naif memory leaks due to "java" new's (lazy java -> C++ translation)

   @todo General review, document it in english and make it conform with Eva specification 1.1
*/

#include "Cadena.h"
#include <cstring>


#define ATOM_ENVOLVER  "\""
#define ATOM_SEPARATOR ","

void Cadena::clear ()
{
    o_str = "";
    o_separ = ",";
    o_lastToken = "";
}

/**
   Returns a "const char *" pointer to the position 'pos' or to the end if pos is invalid
*/
const char * Cadena::c_str (int pos)
{
    if (pos < 0 || pos > length ())
        pos = length ();

    return pos + o_str.c_str ();
}

int Cadena::del (int nchars, int offset)
{
    int lena = length ();
    o_str.erase (offset, nchars);
    return lena - length ();
}

bool Cadena::InitialMatch (const std::string & str)
{
    if (length () < str.length ())
        return false;
    return (0 == o_str.compare (0, str.length (), str));
}

int Cadena::indexOf (const char * chars, int desde)
{
    int tt = desde;

    while (desde < length ())
    {
        for (int ii = 0; ii < strlen (chars); ii ++)
            if (chars[ii] == o_str[desde]) return desde;
        desde ++;
    }
    return -1;
}

int Cadena::replaceMe (const std::string & busca, const std::string & reemp)
{
    // is there something to do ?
    //
    if (busca.length () == 0)
        return 0;

    // do it!
    //
    int pos1 = 0, pos2 = 0;
    int veces = 0;
    std::string novo;

    std::string esto = o_str;

    while (pos1 < length ())
    {
        pos2 = find (busca, pos1);
        if (pos2 == -1) break;  // >>>>>>>
        // found it !
        novo.append (o_str, pos1, pos2 - pos1);   // faster than "... += substr ..."
        novo.append (reemp);
        pos1 = pos2 + busca.length ();
        veces ++;
    }

    if (veces > 0)
    {
        // copy the rest (maybe would be better to use replace instead)
        if (pos1 < length ())
            novo.append (c_str (pos1));
        o_str = novo;
    }
    return veces;
}

std::string Cadena::substrBE (int ini, int fin)
{
    if (ini < 0) ini = 0;
    if (ini > length ()) ini = length ();
    int len = 1+fin-ini;
    if (len < 0) len = 0;
    if (ini+len > length ()) len = length ()-ini;

    return std::string (o_str.substr (ini, len));
}

int Cadena::find (std::string busca, int indx)
{
    return o_str.find (busca, indx);
    /*
    if (busca.length () == 0) return -1;

    int pp = indx;
    while (pp != -1) {
       if ( (-1 == (pp = o_str.indexOf(busca.charAt (0), pp))) ||
            busca.equals (substrBE (pp, pp + busca.length ()-1))
          )
          break;
       pp ++;
    }
    return pp;
    */
}


void Cadena::trimMe ()
{
    // right white spaces and tabs
    while (length () > 0 && (o_str [length ()-1] == ' ' || o_str [length ()-1] == '\t'))
        del (1, length ()-1);

    // left blanks
    while (length () > 0 && (o_str [0] == ' ' || o_str [0] == '\t'))
        del (1);
}

bool Cadena::getToken ()
{
    int wo = indexOf (o_separ.c_str (), 0);

    if (wo < 0)
    {
        o_lastToken = o_str;
        o_str = "";
        return (o_lastToken.length () > 0);
    }

    o_lastToken = o_str.substr (0, wo);
    o_str = o_str.substr (wo+1, length ());
    return true;
}



void Cadena::EnpaquetaMe ()
{
    int tot = 0;
    tot += replaceMe ("\"", "\"\"");
    tot += replaceMe (",", ",");
    if (tot > 0)
        o_str = ATOM_ENVOLVER + o_str + ATOM_ENVOLVER;
}

void Cadena::DesEnpaquetaMe ()
{
    trimMe ();

    if (length () < 2)
        return;

    // take off ""
    //
    if (o_str[0] == ATOM_ENVOLVER[0])
        o_str = substrBE (1, length ());
    if (o_str[length ()-1] == ATOM_ENVOLVER[0])
        o_str = substrBE (0, length ()-2);

    int tot = 0;
    tot += replaceMe ("\"\"", "\"");
}


// bool Cadena::setStrArray (string * slist)
bool Cadena::setVector (std::vector<std::string> & vecStr)
{
    Cadena aux;

    o_str = "";
    for (int ii = 0; ii < vecStr.size (); ii ++)
    {
        aux.setStr (vecStr [ii].c_str ());
        aux.EnpaquetaMe ();
        o_str.append (aux.getStr ());

        if (ii + 1 < vecStr.size ())
            o_str.append (ATOM_SEPARATOR);
    }
    if (o_str == "") o_str = "\"\""; // at least should be [""]
    return true;
}


std::string Cadena::getFirstToken (Cadena & some)
{
    //
    // OJO !! Esta rutina emplea Cadenas !!
    //
    int Cenv=-2, Csep=-2;

    // poblema gordo :
    //       Tenemos que separar los tokens. Estos estn separados
    //       por comas, pero si un token contiene comas(,) (o comillas ",
    //       o espacios al principio)
    //    entonces se pone entre comillas (envoltura) y si el token contiene comillas
    //    stas se doblan (""). En resumen tenemos que buscar o bien una coma o bien
    //       encontrar   las comillas exclusivamente de envoltura.
    //
    some.trimMe ();   // no puede contener espacios al principio!


    Csep = some.find (ATOM_SEPARATOR, 0);  // donde est la coma (,)
    if (Csep == -1)
    {
        // NO HAY COMA "," => un solo token
        Cadena aux (some.o_str);
        aux.DesEnpaquetaMe ();        // unpack it
        some.setStr ("");

        return aux.o_str;
    }

    // ... tenemos una coma en la posicin 'Csep'

    // buscamos la la posible envoltura (")
    Cenv = some.find (ATOM_ENVOLVER, 0);
    if (Cenv == -1 || Cenv > Csep)
    {
        // el problema (") est ms adelante, en otro token o no est
        // p.e.   [blabla,otro,"este tiene problemas"]

        Cadena aux (some.substrBE (0, Csep-1));
        some.o_str = some.substrBE (Csep+1, some.length ());

        aux.DesEnpaquetaMe ();        // unpack it

        return aux.o_str;
    }
    //System.out.println ("segunda fase Csep = [" + Csep + "] Cenv [" + Cenv + "]");

    // ... una comilla antes de la coma 'Csep' (debe ser en la posicin 0 puesto que hemos hecho un trim)
    // "....",
    // "..,..",
    // "..."",...",
    // ASSERT(Cenv == 0)

    // Buscamos la siguiente envoltura vlida saltndonos las dobles comillas [""]
    while (Cenv < some.length ())
    {
        Cenv = some.find (ATOM_ENVOLVER, Cenv+1);
        if (Cenv == -1)
        {
            // no esta bien cerrado .. problemas ... lo enchufamos todo
            Cenv = some.length ();
        }
        if (Cenv+1 >= some.length () ||
                some.o_str [Cenv+1] != ATOM_ENVOLVER[0]) break;
        // era una doble comilla
        Cenv ++;
    }

    Cadena aux (some.substrBE (0, Cenv));
    some.o_str = some.substrBE (Cenv+1, some.length ());
    some.trimMe();
    // todava queda la posibilidad que ahora tengamos un separador!
    if (some.length() > 0 && some.o_str[0] == ATOM_SEPARATOR[0])
        some.del(1);
    aux.DesEnpaquetaMe ();        // unpack it

    return aux.o_str;
}


void Cadena::toVector (std::vector<std::string> & vecStr) const
{
    Cadena queda(o_str.c_str (), o_separ.c_str ());

    vecStr.clear ();

    while (queda.length () > 0)
    {
        std::string quepo = getFirstToken (queda);
        vecStr.push_back (quepo);
    }
}

std::vector<std::string> Cadena::simpleToVector (const char * simpleStringList, const char * separator)
{
    std::vector<std::string> veco;

    simpleToVector (simpleStringList, separator, veco);

    return veco;
}

void Cadena::simpleToVector (const char * simpleStringList, const char * separator, std::vector<std::string> & veco)
{
    Cadena queda(simpleStringList, separator);
    veco.clear ();

    while (queda.getToken ())
        veco.push_back (queda.lastToken ());
}
