/*
Eva library C++ (www.elxala.de)
Copyright (C) 2005  Alejandro Xalabarder Aulet

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _CELTA_CADENA_H_
#define _CELTA_CADENA_H_

#include <vector>
#include <string>

/**
	@class  Cadena
	@author Alejandro Xalabarder
	@date   03.07.2003

   @memo
   string extension class with some methods specialized for parsing
   comma separated strings as Eva specifies

   @todo General review, document it in english and make it conform with Eva specification 1.1
*/
class Cadena
{
public:
    // MEMBERS
    //
    std::string o_str, o_lastToken, o_separ;

    // CONSTRUCTORS
    //
    Cadena (const char * cad = "", const char * sepa = ",");
    Cadena (const std::string & str);

    // METHODS
    //
    void clear ();

    std::string & getStr ();
    int      length ();
    char     at     (int indx);
    char     charAt (int indx);

    /**
       Returns a "const char *" pointer to the position 'pos' or to the end if pos is invalid
    */
    const char * c_str (int pos = 0);

    void setStr   (const char * cad);
    void setSepar (const char * ssepar);
    bool getToken (const char * ssepar);

    int del (int nchars, int offset = 0);
    bool InitialMatch (const std::string & str);

    /*  Comprueba lmites y desde hasta inclusive (sin -1 o +1 !!)
    *   (BE = begin end)
    */
    std::string substrBE (int ini, int fin);

    /*  indexOf any character within the String chars
    */
    int indexOf (const char * chars, int desde = 0);

    /** indexOf the substring
    */
    int indexOfsubstr (const char * substr, int desde = 0);

//     int replaceMe (char que, char por);

    /* Extensin de replace con Strings
        returns the number of times the replace is done
    */
    int replaceMe (const std::string & busca, const std::string & reemp);

    /** Example:

       Cadena cad = new Cadena ("This is \\\\r a %1 \\\\t");

       String [][] mapa = { { "\\\\", "\\" }, { "\\r", "retorno" }, { "%1", "test" } };

       //    original string is : [This is \\r a %1 \\t]
       // and we will replace :
       //    "\\" with "\"
       //    "\r" with "retorno"
       //    "%1" with "test"

       cad.replaceMeOnce (mapa);
       System.out.println (cad.s_str); // will print "This is \r a test"

       NOTE: uses de.elxala.langutil.bigString
    */
    int replaceMeOnce (const char ** mapa, int size);



    /*  Macro substitution, example:

       Cadena template = new Cadena ("Hallo <<name>>, your name (<<name>>) is very <<adjective>>");
       Map mapo = new TreeMap ();

           mapo.put ("name",      "Alejandro");
         mapo.put ("adjective", "feo");
          mapo.put ("adjective", "bonito");

       template.replaceMe (mapo, "<<", ">>");

       System.out.println (template.s_out);

    */
    int replaceMe (const char ** Pairs, int size, std::string & sOpen, std::string & sClose);

    /*  find string a partir de ndice
    */
    int find (std::string busca, int indx = 0);

    void trimMe ();
    bool getToken ();

    std::string & lastToken ();

    void EnpaquetaMe    ();
    void DesEnpaquetaMe ();

    static std::string getFirstToken (Cadena & some);

    /**
       pass the content of a Cadena to an array of chars
       USING ALWAYS! the separator ',' and taking into account the '"' characters

       Example :

          Cadena o = new Cadena ("this,is,\"\"third,element,and last one\"\"");
          // that means the string [this,is,""third,element,and last one""]

          vector<string> arr;

          o.toVector (arr);

          arr[0] = "this"
          arr[1] = "is"
          arr[2] = ""third,element,and last one""

    */
    void toVector (std::vector<std::string> & vecStr) const;

    /**
       pass the content of a vector<string> to a Cadena
       USING ALWAYS! the separator ',' and using '"' character when necesary

       Example :

          vector<string> arr;

          arr.push_back ("this");
          arr.push_back ("is");
          arr.push_back ("third,element,and last one");

          Cadena cad ();

          cad.setVector (arr);

          // the Cadena's content will be [this,is,"third,element,and last one"]
    */
    // bool setStrArray (string slist []);
    bool setVector (std::vector<std::string> & vecStr);


    /**
       pass the content of a "simple string list" that is a string of tokens
       separated by one or more separators (each of size 1) where the token cannot
       content any of the choosen separators.

       Examples :

          string s1 = "this,is,\"\"third,element,and last one\"\"";   //  [this,is,""third,element,and last one""]
          string s2 = "another:possibility/of this";

          vector<string> & l1 = simpleToList (s1, ",");
          vector<string> & l2 = simpleToList (s2, " /:");

          then  l1:
          l1.get (0) => [this]
          l1.get (1) => [is]
          l1.get (2) => [""third]
          l1.get (3) => [element]
          l1.get (4) => [and last one""]

          and l2:
          l2.get (0) => [another]
          l2.get (1) => [possibility]
          l2.get (2) => [of]
          l2.get (3) => [this]
    */
    static std::vector<std::string> simpleToVector (const char * simpleStringList, const char * separator = "");
    static void simpleToVector (const char * simpleStringList, const char * separator, std::vector<std::string> & veco);
};

inline Cadena::Cadena (const std::string & str)
{
    clear ();
    o_str = str;
}

inline Cadena::Cadena (const char * cad, const char * sepa)
{
    clear ();
    o_str = (cad != NULL) ? cad:"";
    o_separ = (sepa != NULL) ? sepa:"";
}


inline std::string & Cadena::getStr ()
{
    return o_str;
}
inline int    Cadena::length   ()
{
    return o_str.length ();
}
inline char   Cadena::at       (int indx)
{
    return o_str.at (indx);
}
inline char   Cadena::charAt   (int indx)
{
    return at (indx);
}

inline void Cadena::setStr   (const char * cad)
{
    o_str = cad;
};
inline void Cadena::setSepar (const char * ssepar)
{
    o_separ = ssepar;
};
inline bool Cadena::getToken (const char * ssepar)
{
    o_separ = ssepar;
    return getToken ();
}

inline int Cadena::indexOfsubstr (const char * substr, int desde)
{
    return o_str.find (substr, desde);
}

inline std::string & Cadena::lastToken ()
{
    return o_lastToken;
}


#endif
