/*
Eva library C++ (www.elxala.de)
Copyright (C) 2005  Alejandro Xalabarder Aulet

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
   Eva.cpp
   author Alejandro Xalabarder

   @todo General review, document it in english and make it conform with Eva specification 1.1
*/

#include "Eva.h"
#include <cstring>

// ...  static ...
EvaLine Eva::NIL_EvaLine;


Eva::Eva (const char * nom, const char * value):
    Nombre(nom)
{
   if (strlen (value) > 0)
      addLine (EvaLine (value));
}

void Eva::clear ()
{
   lis_EvaLine.clear ();
   NIL_EvaLine.clear ();
   Nombre = "";
}

bool Eva::addLine (const EvaLine & nova)
{
   lis_EvaLine.push_back (nova);
   return true;
}

bool Eva::ensureRow (int nrow)
{
   // insert rows ?
   while (nrow >= rows() && addLine ());

   return (nrow < rows());
}

bool Eva::ensureDim (int rows, int cols)
{
   if (false == ensureRow (rows - 1)) return false;
   for (int rr = 0; rr < rows; rr ++)
      get (rr).ensureCol (cols - 1);

   return true;
}

bool Eva::dim (int rows, int cols)
{
   clear ();
   return ensureDim (rows, cols);
}

EvaLine & Eva::get (int nLin)
{
   if (nLin < rows () && nLin >= 0)
      return lis_EvaLine [nLin];
   else {
      NIL_EvaLine.clear ();
      return NIL_EvaLine;
   }
}

int Eva::maxCols ()
{
   int maxx = 0;
   if (rows () > 0)
   {
      for (int rr = 0; rr < lis_EvaLine.size (); rr ++)
         if (lis_EvaLine[rr].cols () > maxx)
            maxx = lis_EvaLine[rr].cols ();
   }
   return maxx;
}

std::string Eva::toString ()
{
   std::string str;

   str = ("<" + Nombre + ">\n");
   for (int ii = 0; ii < rows (); ii ++)
   {
      str.append (lis_EvaLine[ii].toString ());
      str.append ("\n");
   }
   return std::string (str);
}

void Eva::setValue (const char * sVal, int row, int col) {
   if (row < 0) return;

   if (ensureRow (row))
      lis_EvaLine [row].setValue (sVal, col);
}
