/*
Eva library C++ (www.elxala.de)
Copyright (C) 2005  Alejandro Xalabarder Aulet

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _CELTA_EVA_H_
#define _CELTA_EVA_H_

#include "EvaLine.h"

/**
   @class  Eva
   @author Alejandro Xalabarder
   @date   03.07.2003

   @memo
   An Eva is a structure that has a name (almost any text including white spaces)
   and a number of EvaLine's (see EvaLine) which are basically arrays of strings.
   Thus an Eva can define its contents in rows (EvaLines) and columns (index within the EvaLine).

   @todo General review, document it in english and make it conform with Eva specification 1.1

   @example
   <pre>

         // creating a new eva
         //
         Eva nova("my first eva");

         // adding EvaLine's
         //
         nova.addLine (EvaLine("one,two,three"));
         nova.addLine (EvaLine(",,,"));

         // seting values
         //
         nova[1][2] = "tres";
         nova[1][5] = "wrong, will not work";
         nova.setValue ("this is ok", 1, 6); // setValue makes place
         nova.ensureDim (3, 3);
         nova[2][2] = "fin";

         // accesing all contents
         //
         cout << "eva named =" << nova.getName() << "= contains : \n";
         for (int rr = 0; rr < nova.rows(); rr ++)
         {
            cout << rr << ") ";
            for (int cc = 0; cc < nova.cols(rr); cc ++)
               cout << "[" << nova[rr][cc].c_str() << "] ";
            cout << "\n";
         }

         // deserializing (to save into a file or for debug)
         //
         cout << nova.toString ().c_str ();
   </pre>
*/
class Eva
{
public:

    std::string Nombre;

    std::vector<EvaLine> lis_EvaLine;

    Eva (const char * nom = "", const char * value = "");

    static bool isFalseEvaLine (EvaLine & elin)
    {
        return &elin == &NIL_EvaLine;
    }

    void clear ();

    /**
       Aade una nueva lnea a la eva
    */
    bool addLine (const EvaLine & nova);

    bool addLine ()
    {
        return addLine (EvaLine (""));
    }

    bool ensureRow (int nrow);

    std::string & getName ()
    {
        return Nombre;
    }

    /**
       Ensures the row 'rows'-1 and on each row the number the column
       'cols'-1
    */
    bool ensureDim (int rows, int cols);

    /**
       clears and dimension the eva
    */
    bool dim (int rows, int cols);


    /**
       Retorna la lnea si sta existe, sino null
    */
    EvaLine & get (int nLin);

    /**
    *  Retorna el nmero de filas
    */
    int rows ()
    {
        return lis_EvaLine.size ();
    }

    /**
    *  Retorna el nmero de columnas de la fila "fila"
    */
    int cols (int fila)
    {
        if (rows () > fila && fila >= 0)
            return lis_EvaLine[fila].cols ();
        else return 0;
    }

    /**
       Returns the maximum number of columns of all the rows
    */
    int maxCols ();

    /**
    *  Serializa
    */
    std::string toString ();

    /*
    ------------------------------------------------------
       public String [] getStrArray (int nrow) {
          if (nrow < rows ())
             return (String []) ((EvaLine) lis_EvaLine.get (nrow)).arr_Column;
          return null;
       }

       public String getAsText ()
       {
              String ss = "";
              String ret = System.getProperty("line.separator", "\n");

              for (int rr = 0; rr < rows (); rr ++) {
             if (rr != 0)
                   ss += ret;
                for (int cc = 0; cc < cols (rr); cc ++)
                   ss += getValue (rr, cc);
              }
              return ss;
       }

       public String [] getAsArray ()
       {
          String [] array = new String [rows ()];

              for (int rr = 0; rr < rows (); rr ++) {
                String ss = "";
                for (int cc = 0; cc < cols (rr); cc ++)
                   ss += getValue (rr, cc);

                array[rr] = ss;
              }
              return array;
       }

       string & get (int row = 0, int col = 0) {
          if (row >= 0 && row < rows ())
             return (lis_EvaLine[row])[col];
          else {
             EvaLine::NIL_string.clear ();
             return EvaLine::NIL_string;
          }
       }
    */

    /**
       si la row es mayor redimensiona
    */
    void setValue (const char * sVal, int row = 0, int col = 0);

    EvaLine & operator [] (int nrow)
    {
        return get(nrow);
    }

protected:

    static EvaLine NIL_EvaLine;
};

#endif // _CELTA_EVA_H_
