/*
Eva library C++ (www.elxala.de)
Copyright (c) 2006  Alejandro Xalabarder Aulet

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/*
   EvaLayoutManager.cpp

   author Alejandro Xalabarder
   date   17.04.2006
*/


#include "EvaLayoutManager.h"
#include <cstdlib>

#define HEADER_ADAPT      "A"
#define HEADER_EXPAND     "X"
#define EXPAND_HORIZONTAL "-"
#define EXPAND_VERTICAL   "+"

void EvaLayoutManager::addComponent(const char * componentName, Fl_Widget* compWinHandle)
{
   st_RECT rect;
	//GetWindowRect (compWinHandle, &rect);
   rect.left = compWinHandle->x();
   rect.right = compWinHandle->x()+compWinHandle->w();
   rect.top = compWinHandle->y();
   rect.bottom = compWinHandle->y()+compWinHandle->h();

   // add the component to the array
   componentArray.push_back (widgetInfo(componentName, compWinHandle, rect));
   isPrecalculated = false;
}

void EvaLayoutManager::removeComponents()
{
   componentArray.clear ();
   isPrecalculated = false;
}

void EvaLayoutManager::precalculateAll ()
{
   if (isPrecalculated) return;

   Hmargin = std::max(0, atoi (evaLayout[0][1].c_str()));
   Vmargin = std::max(0, atoi (evaLayout[0][2].c_str()));
   Hgap    = std::max(0, atoi (evaLayout[0][3].c_str()));
   Vgap    = std::max(0, atoi (evaLayout[0][4].c_str()));

   Hdim.clear ();
   Hpos.clear ();
   Vdim.clear ();
   Vpos.clear ();
   columnsReparto.clear ();
   rowsReparto.clear ();

   for (int ii = 0; ii < componentArray.size(); ii ++)
      componentArray[ii].isLaidOut = false;

   int cc;
   int rr;

   // compute Vdim
   fijoV = Vmargin;
   for (rr = 0; rr < nRows(); rr ++)
   {
      std::string & heaRow = rowHeader(rr);
      int gap = (rr == 0) ? 0: Vgap;

      Vdim.push_back (0);
      Vpos.push_back (0);
      VextraPos.push_back (0);

      if (heaRow == HEADER_EXPAND)
         rowsReparto.push_back (rr);      // compute later
      else if (heaRow == "" || heaRow == HEADER_ADAPT)
         Vdim[rr] = minHeightOfRow(rr);   // maximum-minimum of the row
      else
         Vdim[rr] = atoi(heaRow.c_str()); // indicated size

      Vpos[rr] = fijoV + gap;
      fijoV += Vdim[rr];
      fijoV += gap;
   }
   fijoV += Vmargin;

   // compute Hdim
   fijoH = Hmargin;
   for (cc = 0; cc < nColumns(); cc ++)
   {
      std::string & heaCol = columnHeader(cc);
      int gap = (cc == 0) ? 0: Hgap;

      Hdim.push_back (0);
      Hpos.push_back (0);
      HextraPos.push_back (0);

      if (heaCol == HEADER_EXPAND)
         columnsReparto.push_back (cc);   // compute later
      else if (heaCol == "" || heaCol == HEADER_ADAPT)
         Hdim[cc] = minWidthOfColumn(cc); // maximum-minimum of the column
      else
         Hdim[cc] = atoi(heaCol.c_str ()); // indicated size

      Hpos[cc] = fijoH + gap;
      fijoH += Hdim[cc];
      fijoH += gap;
   }
   fijoH += Hmargin;

   // finding all components in the layout array
   for (cc = 0; cc < nColumns(); cc ++)
   {
      for (rr = 0; rr < nRows(); rr ++)
      {
         std::string & name = widgetAt(rr, cc);

         int indx = indxComponent (name);
         if (indx == -1) continue;

         widgetInfo & wid = componentArray[indx];

         // set position x,y
         wid.indxPos.left = cc;
         wid.indxPos.top  = rr;

         // set position x2,y2
         int ava = cc;
         while (ava+1 < nColumns() && widgetAt(rr, ava+1) == EXPAND_HORIZONTAL) ava ++;
         wid.indxPos.right = ava;

         ava = rr;
         while (ava+1 < nRows() && widgetAt(ava+1, cc) == EXPAND_VERTICAL) ava ++;
         wid.indxPos.bottom = ava;

         wid.isLaidOut = true;
      }
   }

   isPrecalculated = true;
}

void EvaLayoutManager::doLayout(int totWidth, int totHeight)
{
   precalculateAll ();

   int ii;

   // repartir H
   for (ii = 0; ii < HextraPos.size(); ii ++) HextraPos[ii] = 0;
   int repartH = (totWidth  - fijoH) / ((columnsReparto.size() == 0) ? 1: columnsReparto.size());
   for (ii = 0; ii < columnsReparto.size(); ii ++)
   {
      int indx = columnsReparto[ii];
      Hdim[indx] = repartH;
      for (int res = indx+1; res < nColumns(); res ++)
         HextraPos[res] += repartH;
   }

   // repartir V
   for (ii = 0; ii < VextraPos.size(); ii ++) VextraPos[ii] = 0;
   int repartV = (totHeight - fijoV) / ((rowsReparto.size() == 0) ? 1: rowsReparto.size());
   for (ii = 0; ii < rowsReparto.size(); ii ++)
   {
      int indx = rowsReparto[ii];
      Vdim[indx] = repartV;
      for (int res = indx+1; res < nRows(); res ++)
         VextraPos[res] += repartV;
   }

   for (ii = 0; ii < componentArray.size(); ii ++)
   {
      widgetInfo & wi = componentArray[ii];

      //ShowWindow (wi.Whand, (wi.isLaidOut) ? SW_SHOW: SW_HIDE);
      if(wi.isLaidOut) wi.Whand->show();
      else wi.Whand->hide();

      if (! wi.isLaidOut) continue;

      int x = Hpos[wi.indxPos.left] + HextraPos[wi.indxPos.left];
      int y = Vpos[wi.indxPos.top]  + VextraPos[wi.indxPos.top];
      int dx = 0;
      int dy = 0;
      int mm;
      for (mm = wi.indxPos.left; mm <= wi.indxPos.right; mm ++)
      {
         if (mm != wi.indxPos.left) dx += Hgap;
         dx += Hdim[mm];
      }
      for (mm = wi.indxPos.top; mm <= wi.indxPos.bottom; mm ++)
      {
         if (mm != wi.indxPos.top) dy += Vgap;
         dy += Vdim[mm];
      }

      if (x < 0 || y < 0 || dx < 0 || dy < 0) continue;

      //MoveWindow (wi.Whand, x, y, dx, dy, TRUE);
      //wi.Whand->position(x+dx, y+dy);
      wi.Whand->hide();
      wi.Whand->resize(x, y, dx, dy);
      if(wi.isLaidOut) wi.Whand->show();
      //InvalidateRect(hControl, NULL, TRUE);
   }
}

void EvaLayoutManager::setLayout (Eva layoutInfo)
{
   evaLayout = layoutInfo;

   isPrecalculated = false;
}

void EvaLayoutManager::getWidgets (std::vector<std::string> & vecNames)
{
   for (int rr = 0; rr < nRows(); rr ++)
      for (int cc = 0; cc < nColumns(); cc ++)
      {
         std::string name = widgetAt (rr, cc);
         if (name.size() > 0 && name != EXPAND_HORIZONTAL && name != EXPAND_VERTICAL)
            vecNames.push_back (name);
      }
}

std::string & EvaLayoutManager::columnHeader(int ncol)
{
   return evaLayout[1][ncol + 1];
}

std::string & EvaLayoutManager::rowHeader(int nrow)
{
   return evaLayout[2 + nrow][0];
}

/**
   index in 'componentArray' of the component with name 'compName'
   if the component is not found it returns -1
*/
int EvaLayoutManager::indxComponent (const std::string & compName)
{
   if (compName.size() == 0 || compName == EXPAND_HORIZONTAL || compName == EXPAND_VERTICAL) return -1;
   for (int cc = 0; cc < componentArray.size (); cc ++)
      if (componentArray[cc].name == compName) return cc;

   // + Error msg!!
   return -1;
}

/**
   max-min width of the column
*/
int EvaLayoutManager::minWidthOfColumn (int ncol)
{
   // el componente ms ancho de la columna
   int maxwidth = 0;
   for (int rr = 0; rr < nRows(); rr ++)
   {
      std::string name = widgetAt (rr, ncol);
      int indx = indxComponent (name);
      if (indx == -1) continue;

      // if the widget occupies more than one column do not compute it
      if (widgetAt (rr, ncol+1) == EXPAND_HORIZONTAL) continue;

      widgetInfo & wid = componentArray[indx];
      int width = wid.iniRect.right - wid.iniRect.left;
      maxwidth = std::max (maxwidth, width);
   }
   return maxwidth;
}

/**
   max-min height of the row
*/
int EvaLayoutManager::minHeightOfRow (int nrow)
{
   // el componente ms alto de la columna
   int maxheight = 0;
   for (int cc = 0; cc < nColumns(); cc ++)
   {
      std::string name = widgetAt (nrow, cc);
      int indx = indxComponent (name);
      if (indx == -1) continue;

      // if the widget occupies more than one row do not compute it
      if (widgetAt (nrow+1, cc) == EXPAND_VERTICAL) continue;

      widgetInfo & wid = componentArray[indx];
      int height = wid.iniRect.bottom - wid.iniRect.top;
      maxheight = std::max (maxheight, height);
   }
   return maxheight;
}
