/*
Eva library C++ (www.elxala.de)
Copyright (C) 2005  Alejandro Xalabarder Aulet

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**

	EvaLine.h
	@author Alejandro Xalabarder 03.07.2003 23:35

	03.07.2003 23:36	Pasar a C++
*/

#ifndef _CELTA_EvaLine_H_
#define _CELTA_EvaLine_H_

#include <string>
#include <vector>

#include "Cadena.h"

// utility, returns the atof of a string
#define ATOFS(s) atof(s.c_str())


/**
	@class  EvaLine
	@author Alejandro Xalabarder
	@date   03.07.2003

   @memo
   A EvaLine is a structure that contains an array of strings, the indices of the array
   are called columns because EvaLine represents a single line within a Eva structure.

   @todo General review, document it in english and make it conform with Eva specification 1.1

   @example
   <pre>

         // creating a new eva line
         //
         EvaLine lineOne("id, name, description");

         // accesing all its Eva's
         //
         cout << "the evaline contains the strings : \n";
         for (int ii = 0; ii < lineOne.cols(); ii ++)
            cout << "\"" << lineOne[ii] << "\" ";

         // accesing columns per index
         //
         cout << "\nat index 2 " << lineOne[2];
         lineOne[5] = "five!ERROR";
         lineOne.ensureCol (6);
         lineOne[6] = "now 6 ok";
         lineOne.setValue ("string, with \"special\" characters", 7);

         // deserializing (to save into a file or for debug)
         cout << "\nwhole line [" << lineOne.toString ().c_str () << "]";
   </pre>
*/
class EvaLine
{
public:

    std::vector<std::string> arr_Column;

    EvaLine (const char * sLin = "");

    static bool isFalseString (std::string & str)
    {
        return &str == &NIL_string;
    }

    void clear ();

    /**
    	Carga la lnea a partir de una cadena que contiene los elementos
    	separados por comas (con "" cuando conviene)
    */
    void set (const Cadena & sLin)
    {
        sLin.toVector (arr_Column);
    }

    /**
    */
    void set (const std::vector<std::string> & aLin)
    {
        arr_Column = aLin;
    }


    /**
    	Sets the value on the column specified. If the line has no
    	such column (but it is greater than 0 !) then it will be resized!.

    	This can be also achived by get(col) or operator []

    	EvaLinevar.setValue ("item", 10);	// (a)
    	EvaLinevar.get (10) = "item";       // (b)
    	EvaLinevar[10] = "item";            // (c)

    	(b) and (c) are identical but only will set the value if the
    	index exists previously while (a) will create the column if needed.

    	@param sVal Value to set
    	@param ncol Number of the column to set the value
    */
    void setValue (const char * sVal, int ncol);

    bool ensureCol (const int ncol);

    bool addValue (const char * sVal = "");

    /**
    	Retorna la lnea en una nueva cadena en donde los elementos estn
    	separados por comas (con "" cuando conviene)
    */
    void getCadena (Cadena & cad)
    {
        cad.setVector (arr_Column);
    }

    /**
    	Retorna la variable de la columna 'ncol' o null
    */
    std::string & get (const int ncol = 0);

    /**
    *    Facilidad que retorna el nmero de columnas
    */
    int cols ()
    {
        return arr_Column.size ();
    }

    /**
    *	Serializa (slo salida)
    */
    std::string toString ();

    std::string & operator [] (const int ncol)
    {
        return get(ncol);
    }

protected:

    static std::string NIL_string;
};

#endif
