/*
Eva library C++ (www.elxala.de)
Copyright (C) 2005  Alejandro Xalabarder Aulet

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _CELTA_EVAUNIT_H_
#define _CELTA_EVAUNIT_H_

#include "Eva.h"

/**
	@class  EvaUnit
	@author Alejandro Xalabarder
	@date   03.07.2003

   @memo
   A EvaUnit is a structure that has a name (almost any text including white spaces)
   and a set of Eva's. Usually will be stored in a text file of format Eva with more
   EvaUnit's

   @todo General review, document it in english and make it conform with Eva specification 1.1

   @example
   <pre>

         // creating a new evanit
         //
         EvaUnit eunit("my first unit");

         // adding eva's
         //
         eunit.add (Eva("unaEva"));
         eunit.add (Eva("secondEva"));

         // accesing all its Eva's
         //
         cout << "unit " << eunit.getName() << " contains the evas : ";
         for (int ii = 0; ii < eunit.size(); ii ++)
            cout << eunit[ii].getName() << " ";

         // accesing Eva per name
         //
         Eva & ptEva1 = eunit.getEva("unaEva");
         Eva & ptEva2 = eunit["unaEva"];

         // deserializing (to save into a file or for debug)
         cout << eunit.toString ().c_str ();
   </pre>
*/
class EvaUnit
{
	public:

	std::string      Nombre;
	std::vector<Eva> lis_Evas;

	EvaUnit (const char * nom = "");

	static bool isFalseEva (Eva & eva) { return &eva == &NIL_Eva; }

   std::string & getName () { return Nombre; }

	/**
	*	Aade una nueva eva a la unidad
	*/
	bool add (const Eva & nova);

	/**
	*	Retorna el nmero de Evas
	*/
	int size ();

	void clear ();

	/**
	*	Retorna el ndice de la eva con el nombre Nom (si existe)
	*	en caso contrario retorna -1
	*/
	int indxEva (const char * Nom);

	/**
	*	Serializa (slo salida)
	*/
	std::string toString ();

	/**
		Retorna la eva de nombre "nameEva" o NIL_Eva si sta no existe.
		@param nameEva Nombre de la eva
	*/
	Eva & getEva (const char * nameEva);

	/**
		Returns true if the eva with name 'nameEva' exists, if not the try to create it and
		return true if succesful otherwise returns false.
	*/
	bool ensureEva (const char * nameEva);

	/**
		gets the eva named 'evaName' if exists in the EvaUnit if not then it is
		created.

		Note : this method returns an Eva object ALWAYS!
		       Even in the case that the EvaUnit is of type ReadOnly,
		       in this case the returned Eva is a dummy one (created but not referenced in the EvaUnit).
	*/
	Eva & getSomeHowEva (const char * evaName);

	std::string & getValue (const char * nameEva, int nrow = 0, int ncol = 0);

	/**
		sets the value 'value' in row and col specified of the Eva with name 'evaName'
		if the eva does not exists or row or col it will try to create it
	*/
	void setValue (const char * evaName, const char * value, int nrow = 0, int ncol = 0);

   Eva & operator [] (const int indxEva);
   Eva & operator [] (const char *evaName);

protected:

	static Eva NIL_Eva;  // dummy element to return on invalid index
};

inline EvaUnit::EvaUnit (const char * nom)
   : Nombre(nom)
{
}

inline int EvaUnit::size ()
{
	return lis_Evas.size ();
}

#endif // _CELTA_EVAUNIT_H_
