#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Text_Editor.H>
#include <FL/Fl_Table.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Box.H>

#include <FL/Fl_Ask.H>

#include "evaLayout/EvaLayoutManager.h"
#include "evaLayout/EvaLine.h"
#include "evaLayout/EvaUnit.h"
#include "evaLayout/EvaFile.h"

// EVA Layout --------------------------------
//
//  variables for this sample
//

class MyWin: public Fl_Double_Window
{
    public:
    typedef MyWin THISCLASS;
    EvaLayoutManager gaston;  // the EvaLayoutManager object
    EvaUnit layoutArray;
    int currentLayIndx;             // # of current layout

    Fl_Table *lista;
    Fl_Text_Editor_Buffered *memo;
    Fl_Choice *combo;

    Fl_Button *boton1;
    Fl_Button *boton2;
    Fl_Button *boton3;
    Fl_Button *boton4;

    Fl_Input *edit1;
    Fl_Input *edit2;
    Fl_Input *edit3;

    Fl_Box *label1;
    Fl_Box *label2;
    Fl_Box *label3;
    Fl_Box *label4;

    MyWin(int x, int y, int w, int h, char *L=0):Fl_Double_Window(x,y,w,h,L)
    {
        currentLayIndx = 0;
        lista = new Fl_Table(10,10, 50, 50);
        lista->end();
        memo = new Fl_Text_Editor_Buffered(10, 60, 50, 50);
        combo = new Fl_Choice(10, 150, 50, 25);

        boton1 = new Fl_Button(10, 180, 100, 25, "Change Layout");
        THISMBACK(boton1, switchLayout);
        boton2 = new Fl_Button(10, 210, 100, 25, "boton2");
        boton3 = new Fl_Button(10, 230, 100, 25, "boton3");
        boton4 = new Fl_Button(10, 260, 100, 25, "boton4");

        edit1 = new Fl_Input(10, 300, 50, 25);
        edit2 = new Fl_Input(10, 330, 50, 25);
        edit3 = new Fl_Input(10, 360, 50, 25);

        label1 = new Fl_Box(300, 10, 50, 25, "label1");
        label2 = new Fl_Box(300, 50, 50, 25, "label2");
        label3 = new Fl_Box(300, 80, 50, 25, "label3");
        label4 = new Fl_Box(300, 120, 50, 25, "label4");
    }
    void resize(int x, int y, int w, int h)
    {
        Fl_Double_Window::resize(x,y,w,h);
        showLayout();
        redraw();
//        for(int i=children()-1; i >=0; i--)
//        {
//            Fl_Widget *w = child(i);
//            printf("%s %d:%d:%d:%d\n", w->classId(), w->x(), w->y(), w->w(), w->h());
//        }
    }
    void prepareLayout ();
    void linkLayout    ();
    void showLayout    (bool forceInvalidate = false);
    void switchLayout  ();
};


void MyWin::prepareLayout ()
{
   // EVA Layout --------------------------------
   // Load layoutInfo in two different ways :
   //
   //    If the file WinLayouts.eva is found then the layout Info's
   //    contained in the unit #data# are loaded. Otherwise
   //    we build here two harcoded Eva layout Info's.
   //

   EvaFile efi;
   if (! efi.load(layoutArray, "ConfigLayouts.eva", "data"))
   {
      fl_alert("Working with two harcoded layoutInfo's\nbecause file ConfigLayouts.eva is not found");

      // create two Eva's on this EvaUnit
      //
      layoutArray.add (Eva ("first"));
      layoutArray.add (Eva ("second"));

      // get the references to the Eva's to fill them
      // (note that eunit[0] and eunit[1] could be used as well)
      //
      Eva & layI1 = layoutArray["first"];

      layI1.addLine (EvaLine ("EvaLayout, 10, 10, 5, 5"));
      layI1.addLine (EvaLine (" zzz,         ,   X      ,  A     ,"));
      layI1.addLine (EvaLine ("    , label1  , edit1    , -      ,"));
      layI1.addLine (EvaLine ("    , label2  , edit2    , -      ,"));
      layI1.addLine (EvaLine ("    ,         , memo     , boton1 ,"));
      layI1.addLine (EvaLine ("    ,         ,   +      , boton2 ,"));
      layI1.addLine (EvaLine ("   X ,         ,   +      ,       ,"));

      Eva & layI2 = layoutArray["second"];

      layI2.addLine (EvaLine ("EvaLayout, 10, 10, 5, 5"));
      layI2.addLine (EvaLine ("xxx,    A     ,    A    , X       ,  "));
      layI2.addLine (EvaLine ("  A, boton1   , label1  , edit1   ,  "));
      layI2.addLine (EvaLine ("  A, boton2   , label2  , edit2   ,  "));
      layI2.addLine (EvaLine ("  A,          , label3  , edit3   ,  "));
      layI2.addLine (EvaLine ("  A, memo     ,   -     ,    -    ,  "));
      layI2.addLine (EvaLine ("  X,  +       ,         ,         ,  "));

      // to export the harcoded EvaUnit ...
      //
      // efi.saveSingleUnit (eunit, "hardcoded.eva");
   }

   if (layoutArray.size () == 0)
   {
      fl_alert("Error there are no layouts loaded!");
      exit(0);
      //EndDialog (hDlg, LOWORD(wParam));
   }
}

void MyWin::linkLayout ()
{
   // ---- EvaLayout: Associate all components with the logic names
   //                 of the layout info or layouts infos. Usually made only once
   //                 but all components from all possible layouts has to be declared here.
   //
   gaston.addComponent("lista",   lista);
   gaston.addComponent("memo",    memo);
   gaston.addComponent("combo",   combo);

   gaston.addComponent("boton1",  boton1);
   gaston.addComponent("boton2",  boton2);
   gaston.addComponent("boton3",  boton3);
   gaston.addComponent("boton4",  boton4);

   gaston.addComponent("edit1",   edit1);
   gaston.addComponent("edit2",   edit2);
   gaston.addComponent("edit3",   edit3);

   gaston.addComponent("label1",  label1);
   gaston.addComponent("label2",  label2);
   gaston.addComponent("label3",  label3);
   gaston.addComponent("label4",  label4);
   // ================================================================
}

/**
   // ---- EvaLayout: method for MFC to recalculate the layout
   //
   call this function after setting a new layout
   or resizing the window
*/
void MyWin::showLayout (bool forceInvalidate)
{
   // ---- EvaLayout: do the real job
   //
   gaston.doLayout(w(), h());

   // for example: just invalidate sometimes to avoid flickering
   //              if Invalidate is never called then in some situations
   //              the components are wrong painted
//   static int sometimes = 0;
//   if (forceInvalidate || sometimes ++ > 40)
//   {
//      sometimes = 0;
//      //InvalidateRect(hDlg, &rc, true);
//      //resize(rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top);
//   }
}

void MyWin::switchLayout ()
{
   // ---- EvaLayout: set the layout info into the manager. If only one layout is needed
   //                 it is enough to do it just once. But if more layouts are to be handled
   //                 then this is the proper method to switch the layout

   // switch to next layout ...
   //
   if (currentLayIndx >= layoutArray.size())
      currentLayIndx = 0;
   gaston.setLayout(layoutArray[currentLayIndx ++]);

   showLayout (true);
}


int main(int argc, char * argv[])
{

    MyWin win(100,100, 400, 400, "Eva Layout Demo");
    win.end();
    win.prepareLayout ();
    win.linkLayout();
    win.switchLayout();
    win.resizable(win);
    win.show(argc, argv);
    Fl::run();
    return 0;
}
